@@ -4,33 +4,34 @@ import logging
 
 from datetime import datetime
 from time import time
+from typing import Any
 
 from .const import VERSION, VERSION_APP
-from .exception_classes import LoginError, NoAdminError
+from .exception_classes import ErrorMSG, LoginError, NoAdminError
 from .gql_handler import GQLHandler
 from .pyxplora import PyXplora
-from .status import LocationType, NormalStatus, WatchOnlineStatus
+from .status import LocationType, NormalStatus, UserContactType, WatchOnlineStatus
 
 _LOGGER = logging.getLogger(__name__)
 
-_LIST_DICT: list[dict[str, any]] = []
+_LIST_DICT: list[dict[str, Any]] = []
 
 
 class PyXploraApi(PyXplora):
     def __init__(
         self,
-        countrycode: str,
-        phoneNumber: str,
-        password: str,
-        userLang: str,
-        timeZone: str,
+        countrycode: str = "",
+        phoneNumber: str = "",
+        password: str = "",
+        userLang: str = "",
+        timeZone: str = "",
         childPhoneNumber: list[str] = [],
-        wuid: str | list = None,
-        email: str = None,
+        wuid: str | list | None = None,
+        email: str | None = None,
     ) -> None:
         super().__init__(countrycode, phoneNumber, password, userLang, timeZone, childPhoneNumber, wuid, email)
 
-    def _login(self, forceLogin: bool = False) -> dict[any, any]:
+    def _login(self, forceLogin: bool = False, signup: bool = True) -> dict[Any, Any]:
         if not self._isConnected() or self._hasTokenExpired() or forceLogin:
             try:
                 self._logoff()
@@ -41,6 +42,7 @@ class PyXploraApi(PyXplora):
                     self._userLang,
                     self._timeZone,
                     self._email,
+                    signup,
                 )
                 if self._gqlHandler:
                     retryCounter = 0
@@ -54,7 +56,7 @@ class PyXploraApi(PyXplora):
                             self.error_message = err.message
                         except Exception:
                             if retryCounter == self.maxRetries + 2:
-                                self.error_message = "Cannot connect to the server."
+                                self.error_message = ErrorMSG.SERVER_ERR
                             else:
                                 pass
 
@@ -70,24 +72,26 @@ class PyXploraApi(PyXplora):
                 self._logoff()
         return self._issueToken
 
-    def init(self, forceLogin: bool = False) -> None:
-        token = self._login(forceLogin)
-        if token:
-            if token.get("user", {}):
-                if not self._childPhoneNumber:
-                    self.watchs = token.get("user", {}).get("children", _LIST_DICT)
-                else:
-                    for watch in token.get("user", {}).get("children", _LIST_DICT):
-                        if watch["ward"]["phoneNumber"] in self._childPhoneNumber:
-                            self.watchs.append(watch)
-                self.user = token.get("user", {})
-                return
-        raise LoginError(self.error_message)
+    def init(self, forceLogin: bool = False, signup: bool = True) -> None:
+        token = self._login(forceLogin, signup)
+        if signup:
+            if token:
+                if token.get("user", {}):
+                    if not self._childPhoneNumber:
+                        self.watchs = token.get("user", {}).get("children", _LIST_DICT)
+                    else:
+                        for watch in token.get("user", {}).get("children", _LIST_DICT):
+                            if watch["ward"]["phoneNumber"] in self._childPhoneNumber:
+                                self.watchs.append(watch)
+                    self.user = token.get("user", {})
+                    return
+            raise LoginError(self.error_message)
+        return
 
     def version(self) -> str:
         return "{0}-{1}".format(VERSION, VERSION_APP)
 
-    def setDevices(self, ids: list = []) -> list[dict[str, any]]:
+    def setDevices(self, ids: list = []) -> list[str]:
         return self._setDevices(ids)
 
     def _setDevices(self, ids: list = []) -> list[str]:
@@ -116,23 +120,23 @@ class PyXploraApi(PyXplora):
             self.device[wuid]["getWatchState"] = self.getWatchState(wuid=wuid, watches=self.device[wuid]["getWatches"])
             d = datetime.now()
             dt = datetime(year=d.year, month=d.month, day=d.day)
-            self.device[wuid]["getWatchUserSteps"] = self.getWatchUserSteps(wuid=wuid, date=dt.timestamp())
+            self.device[wuid]["getWatchUserSteps"] = self.getWatchUserSteps(wuid=wuid, date=int(dt.timestamp()))
             self.device[wuid]["getWatchOnlineStatus"] = self.getWatchOnlineStatus(wuid=wuid)
             self.device[wuid]["getWatchUserIcons"] = self.getWatchUserIcons(wuid=wuid)
             self.device[wuid]["getWatchUserXcoins"] = self.getWatchUserXcoins(wuid=wuid)
         return wuids
 
     ##### Contact Info #####
-    def getWatchUserContacts(self, wuid: str) -> list[dict[str, any]]:
+    def getWatchUserContacts(self, wuid: str) -> list[dict[str, Any]]:
         retryCounter = 0
-        dataOk: list[dict[str, any]] = []
-        contacts_raw: dict[str, any] = {}
-        contacts: list[dict[str, any]] = []
+        dataOk: list[dict[str, Any]] = []
+        contacts_raw: dict[str, Any] = {}
+        contacts: list[dict[str, Any]] = []
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
                 contacts_raw = self._gqlHandler.getWatchUserContacts(wuid)
-                _contacts: dict[str, any] = contacts_raw.get("contacts", {})
+                _contacts: dict[str, Any] = contacts_raw.get("contacts", {})
                 if not _contacts:
                     dataOk.append({})
                     return contacts
@@ -167,11 +171,11 @@ class PyXploraApi(PyXplora):
                 self.delay(self.retryDelay)
         return contacts
 
-    def getWatchAlarm(self, wuid: str) -> list[dict[str, any]]:
+    def getWatchAlarm(self, wuid: str) -> list[dict[str, Any]]:
         retryCounter = 0
-        dataOk: list[dict[str, any]] = []
-        alarms_raw: dict[str, any] = {}
-        alarms: list[dict[str, any]] = []
+        dataOk: list[dict[str, Any]] = []
+        alarms_raw: dict[str, Any] = {}
+        alarms: list[dict[str, Any]] = []
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
@@ -199,11 +203,11 @@ class PyXploraApi(PyXplora):
                 self.delay(self.retryDelay)
         return alarms
 
-    def loadWatchLocation(self, wuid: str = "", withAsk: bool = True) -> dict[str, any]:
+    def loadWatchLocation(self, wuid: str = "", withAsk: bool = True) -> dict[str, Any]:
         retryCounter = 0
-        dataOk: list[dict[str, any]] = []
-        location_raw: dict[str, any] = {}
-        watch_location: dict[str, any] = {}
+        dataOk: dict[str, Any] = {}
+        location_raw: dict[str, Any] = {}
+        watch_location: dict[str, Any] = {}
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
@@ -216,7 +220,7 @@ class PyXploraApi(PyXplora):
                 )
                 if not _watchLastLocate:
                     return watch_location
-                _tm = 31532399 if _watchLastLocate.get("tm") is None else _watchLastLocate.get("tm")
+                _tm: float = 31532399 if _watchLastLocate.get("tm", None) is None else _watchLastLocate.get("tm", 31532399)
                 _lat = "0.0" if _watchLastLocate.get("lat") is None else _watchLastLocate.get("lat")
                 _lng = "0.0" if _watchLastLocate.get("lng") is None else _watchLastLocate.get("lng")
                 _rad = -1 if _watchLastLocate.get("rad") is None else _watchLastLocate.get("rad")
@@ -258,11 +262,11 @@ class PyXploraApi(PyXplora):
         return watch_location
 
     def getWatchBattery(self, wuid: str) -> int:
-        watch_b: dict[str, any] = self.loadWatchLocation(wuid=wuid)
+        watch_b: dict[str, Any] = self.loadWatchLocation(wuid=wuid)
         return int(watch_b.get("watch_battery", -1))
 
     def getWatchIsCharging(self, wuid: str) -> bool:
-        watch_c: dict[str, any] = self.loadWatchLocation(wuid=wuid)
+        watch_c: dict[str, Any] = self.loadWatchLocation(wuid=wuid)
         if watch_c.get("watch_charging", False):
             return True
         return False
@@ -295,12 +299,12 @@ class PyXploraApi(PyXplora):
         # bug?
         return (self._gqlHandler.unReadChatMsgCount(wuid)).get("unReadChatMsgCount", -1)
 
-    def getWatchChats(self, wuid: str, offset: int = 0, limit: int = 100, msgId: str = "") -> list[dict[str, any]]:
+    def getWatchChats(self, wuid: str, offset: int = 0, limit: int = 100, msgId: str = "") -> list[dict[str, Any]]:
         # bug?
         retryCounter = 0
-        dataOk: list[dict[str, any]] = []
-        chats_raw: dict[str, any] = {}
-        chats: list[dict[str, any]] = []
+        dataOk: list[dict[str, Any]] = []
+        chats_raw: dict[str, Any] = {}
+        chats: list[dict[str, Any]] = []
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
@@ -339,24 +343,24 @@ class PyXploraApi(PyXplora):
         return chats
 
     ##### Watch Location Info #####
-    def getWatchLastLocation(self, wuid: str, withAsk: bool = False) -> dict[str, any]:
+    def getWatchLastLocation(self, wuid: str, withAsk: bool = False) -> dict[str, Any]:
         _loadWatchLocation = self.loadWatchLocation(wuid=wuid, withAsk=withAsk)
         if isinstance(_loadWatchLocation, dict):
             return _loadWatchLocation.get("watch_last_location", {})
-        if not _loadWatchLocation:
-            return {}
-        for loadWatchLocation in _loadWatchLocation:
-            return loadWatchLocation.get("watch_last_location", {})
+        # if not _loadWatchLocation:
+        #     return {}
+        # for loadWatchLocation in _loadWatchLocation:
+        #     return loadWatchLocation.get("watch_last_location", {})
         return {}
 
-    def getWatchLocate(self, wuid: str) -> dict[str, any]:
+    def getWatchLocate(self, wuid: str) -> dict[str, Any]:
         _loadWatchLocation = self.loadWatchLocation(wuid=wuid)
         if isinstance(_loadWatchLocation, dict):
             return _loadWatchLocation
-        if not _loadWatchLocation:
-            return {}
-        for loadWatchLocation in _loadWatchLocation:
-            return loadWatchLocation
+        # if not _loadWatchLocation:
+        #     return {}
+        # for loadWatchLocation in _loadWatchLocation:
+        #     return loadWatchLocation
         return {}
 
     def getWatchLocateType(self, wuid: str) -> str:
@@ -368,11 +372,11 @@ class PyXploraApi(PyXplora):
     def getWatchSafeZoneLabel(self, wuid: str) -> str:
         return self.getWatchLocate(wuid).get("safeZoneLabel", "")
 
-    def getWatchSafeZones(self, wuid: str) -> list[dict[str, any]]:
+    def getWatchSafeZones(self, wuid: str) -> list[dict[str, Any]]:
         retryCounter = 0
-        dataOk: list[dict[str, any]] = []
-        safeZones_raw: dict[str, any] = {}
-        safe_zones: list[dict[str, any]] = []
+        dataOk: list[dict[str, Any]] = []
+        safeZones_raw: dict[str, Any] = {}
+        safe_zones: list[dict[str, Any]] = []
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
@@ -408,11 +412,11 @@ class PyXploraApi(PyXplora):
         return self._gqlHandler.askWatchLocate(wuid).get("askWatchLocate", False)
 
     ##### Feature #####
-    def getSilentTime(self, wuid: str) -> list[dict[str, any]]:
+    def getSilentTime(self, wuid: str) -> list[dict[str, Any]]:
         retryCounter = 0
-        dataOk: list[dict[str, any]] = []
-        silentTimes_raw: dict[str, any] = {}
-        school_silent_mode: list[dict[str, any]] = []
+        dataOk: list[dict[str, Any]] = []
+        silentTimes_raw: dict[str, Any] = {}
+        school_silent_mode: list[dict[str, Any]] = []
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
@@ -568,19 +572,19 @@ class PyXploraApi(PyXplora):
         raise NoAdminError()
 
     def getFollowRequestWatchCount(self) -> int:
-        c: dict[str, any] = self._gqlHandler.getFollowRequestWatchCount()
+        c: dict[str, Any] = self._gqlHandler.getFollowRequestWatchCount()
         return c.get("followRequestWatchCount", 0)
 
-    def getWatches(self, wuid: str) -> dict[str, any]:
+    def getWatches(self, wuid: str) -> dict[str, Any]:
         retryCounter = 0
-        dataOk: dict[str, any] = {}
-        watches_raw: dict[str, any] = {}
-        watches: dict[str, any] = {}
+        dataOk: dict[str, Any] = {}
+        watches_raw: dict[str, Any] = {}
+        watches: dict[str, Any] = {}
         while not dataOk and (retryCounter < self.maxRetries + 2):
             retryCounter += 1
             try:
                 watches_raw = self._gqlHandler.getWatches(wuid)
-                _watches: list[dict[str, any]] = watches_raw.get("watches", [])
+                _watches: list[dict[str, Any]] = watches_raw.get("watches", [])
                 if not _watches:
                     return watches
                 for watch in _watches:
@@ -598,45 +602,46 @@ class PyXploraApi(PyXplora):
                 self.delay(self.retryDelay)
         return watches
 
-    def getSWInfo(self, wuid: str, watches: dict[str, any] = {}) -> dict[str, any]:
-        wqr: dict[str, any] = watches if watches else self.getWatches(wuid=wuid)
+    def getSWInfo(self, wuid: str, watches: dict[str, Any] = {}) -> dict[str, Any]:
+        wqr: dict[str, Any] = watches if watches else self.getWatches(wuid=wuid)
         qrCode: str = wqr.get("qrCode", "=")
         return self._gqlHandler.getSWInfo(qrCode.split("=")[1])
 
-    def getWatchState(self, wuid: str, watches: dict[str, any] = {}) -> dict[str, any]:
-        wqr: dict[str, any] = watches if watches else self.getWatches(wuid=wuid)
+    def getWatchState(self, wuid: str, watches: dict[str, Any] = {}) -> dict[str, Any]:
+        wqr: dict[str, Any] = watches if watches else self.getWatches(wuid=wuid)
         qrCode: str = wqr.get("qrCode", "=")
         try:
             return self._gqlHandler.getWatchState(qrCode=qrCode.split("=")[1])
         except Exception as err:
-            print(err)
+            _LOGGER.debug(err)
+            return {}
 
-    def conv360IDToO2OID(self, qid: str, deviceId: str) -> dict[str, any]:
+    def conv360IDToO2OID(self, qid: str, deviceId: str) -> dict[str, Any]:
         return self._gqlHandler.conv360IDToO2OID(qid, deviceId)
 
-    def campaigns(self, id: str, categoryId: str) -> dict[str, any]:
+    def campaigns(self, id: str, categoryId: str) -> dict[str, Any]:
         return self._gqlHandler.campaigns(id, categoryId)
 
     def getCountries(self) -> list[dict[str, str]]:
-        countries: dict[str, any] = self._gqlHandler.countries()
+        countries: dict[str, Any] = self._gqlHandler.countries()
         return countries.get("countries", {})
 
-    def getWatchLocHistory(self, wuid: str, date: int, tz: str, limit: int) -> dict[str, any]:
+    def getWatchLocHistory(self, wuid: str, date: int, tz: str, limit: int) -> dict[str, Any]:
         return self._gqlHandler.getWatchLocHistory(wuid, date, tz, limit)
 
-    def watchesDynamic(self) -> dict[str, any]:
+    def watchesDynamic(self) -> dict[str, Any]:
         return self._gqlHandler.watchesDynamic()
 
-    def watchGroups(self, id: str = "") -> dict[str, any]:
+    def watchGroups(self, id: str = "") -> dict[str, Any]:
         return self._gqlHandler.watchGroups(id)
 
-    def familyInfo(self, wuid: str, watchId: str, tz: str, date: int) -> dict[str, any]:
+    def familyInfo(self, wuid: str, watchId: str, tz: str, date: int) -> dict[str, Any]:
         return self._gqlHandler.familyInfo(wuid, watchId, tz, date)
 
-    def avatars(self, id: str) -> dict[str, any]:
+    def avatars(self, id: str) -> dict[str, Any]:
         return self._gqlHandler.avatars(id)
 
-    def getWatchUserSteps(self, wuid: str, date: int) -> dict[str, any]:
+    def getWatchUserSteps(self, wuid: str, date: int) -> dict[str, Any]:
         userSteps = self._gqlHandler.getWatchUserSteps(wuid=wuid, tz=self._timeZone, date=date)
         if not userSteps:
             return {}
@@ -647,12 +652,12 @@ class PyXploraApi(PyXplora):
 
     # start tracking for 30min
     def getStartTrackingWatch(self, wuid: str) -> int:
-        data: dict[str, any] = self._gqlHandler.getStartTrackingWatch(wuid)
+        data: dict[str, Any] = self._gqlHandler.getStartTrackingWatch(wuid)
         return data.get("startTrackingWatch", -1)
 
     # stop tracking from getStartTrackingWatch
     def getEndTrackingWatch(self, wuid: str) -> int:
-        data: dict[str, any] = self._gqlHandler.getEndTrackingWatch(wuid)
+        data: dict[str, Any] = self._gqlHandler.getEndTrackingWatch(wuid)
         return data.get("endTrackingWatch", -1)
 
     def addStep(self, step: int) -> bool:
@@ -666,3 +671,15 @@ class PyXploraApi(PyXplora):
     def getAppVersion(self):
         data = self._gqlHandler.getAppVersion()
         return data
+
+    def checkEmailOrPhoneExist(
+        self, type: UserContactType, email: str = "", countryCode: str = "", phoneNumber: str = ""
+    ) -> bool:
+        data = self._gqlHandler.checkEmailOrPhoneExist(type, email, countryCode, phoneNumber)
+        return data.get("checkEmailOrPhoneExist", False)
+
+    def modifyContact(
+        self, contactId: str, isAdmin: bool | None = None, contactName: str = "", fileId: str = ""
+    ) -> dict[str, Any]:
+        data = self._gqlHandler.modifyContact(contactId, isAdmin, contactName, fileId)
+        return data
